@@ -18,14 +18,14 @@ import os
 import sys
 import re
 from os import path
-from calculate.lib.datavars import Variable,VariableError,ReadonlyVariable
+from calculate.lib.datavars import Variable, VariableError, ReadonlyVariable
 from calculate.lib.utils.portage import isPkgInstalled
-from calculate.lib.utils.files import process,STDOUT,getProgPath
+from calculate.lib.utils.files import process, STDOUT, getProgPath, readFile
 from calculate.lib.utils.common import (getVideoFromXorgLog,
                 getVideoFromXorgConf, getVideoFromCmdLine,
                 getAvailableVideo, getValueFromCmdLine,
                 getCompositeFromXorgconf, getVideoFromModules,
-                getVideoFromVendor,getInstalledVideo)
+                getVideoFromVendor, getInstalledVideo)
 from calculate.install.distr import DistributiveError
 
 from calculate.lib.cl_lang import setLocalTranslate
@@ -36,6 +36,14 @@ class VideoVariable(Variable):
     Video variables not using for flash installation
     """
     xorg_need = True
+    default_video = "default"
+    driver_names = {'nvidia': _("NVidia proprietary video driver"),
+                    'fglrx': _("ATI proprietary video driver"),
+                    'intel': _("Intel video driver"),
+                    'radeon': _("ATI open source video driver"),
+                    'nouveau': _("NVidia open source video driver"),
+                    'default': _("X.Org Server auto detection")
+    }
 
     def uncompatible(self):
         """
@@ -44,7 +52,7 @@ class VideoVariable(Variable):
         if self.Get('os_install_root_type') == 'flash':
             return \
                 _("Video configuration unavailable for Flash install")
-        if self.Get('os_install_x11_server_set') == 'no' and self.xorg_need:
+        if self.Get('os_install_x11_server_set') == 'off' and self.xorg_need:
             return \
                 _("This distribution does not provide a Xorg server")
         return ""
@@ -103,8 +111,7 @@ class VariableOsInstallX11VideoAvailable(VideoVariable):
     Get available (already installed or installable drivers
     """
     type = "list"
-    supported = ["nvidia","vboxvideo","sis","nv",
-                 "fglrx","nouveau","intel","ati","radeon"]
+    supported = ["nvidia", "fglrx", "nouveau", "intel", "radeon"]
 
     def get(self):
         image = self.Get('cl_image')
@@ -113,18 +120,24 @@ class VariableOsInstallX11VideoAvailable(VideoVariable):
                 try:
                     distrPath = image.getDirectory()
                     if isPkgInstalled('xorg-server',prefix=distrPath):
-                        return sorted(filter(self.supported.__contains__,
-                               getAvailableVideo(prefix=distrPath)))+['other']
+                        return (sorted(filter(self.supported.__contains__,
+                                getAvailableVideo(prefix=distrPath))) +
+                                [self.default_video])
                 except DistributiveError as e:
                     pass
         return []
 
+    def humanReadable(self):
+        return map(lambda x: self.driver_names.get(x, x), self.Get())
+
+
 class VariableOsX11KmsVideoDrv(ReadonlyVariable):
     """
     Список KMS драйверов
     """
     type = "list"
-    value = ["radeon","i915","intel","nouveau","ati"]
+    value = ["radeon", "intel", "nouveau"]
+
 
 class VariableOsInstallX11VideoDrv(VideoVariable):
     """
@@ -136,54 +149,82 @@ class VariableOsInstallX11VideoDrv(VideoVariable):
 
     def init(self):
         self.help = _("set the video driver")
-        self.label = _("{0} video driver").format("Xorg")
+        self.label = _("Video driver")
 
     def choice(self):
         """Get available (already installed or installable drivers"""
-        return self.Get('os_install_x11_video_available')
+        if self.Get('os_install_x11_server_set') == 'on':
+            values = self.Get('os_install_x11_video_available')
+        else:
+            values = self.Get('os_x11_kms_video_drv') + [self.default_video]
+        return map(lambda x: (x, self.driver_names.get(x, x)), values)
 
     def get(self):
-        # get available videodriver list from install or configure distributive
-        list_video = self.Choice('os_install_x11_video_drv')
-        if not list_video:
-            return "other"
-        # if type system is usb-hdd then get detect video driver
-        if self.Get('os_install_root_type') == 'usb-hdd':
-            methods = ((getVideoFromModules,()),
-                       (getVideoFromCmdLine,()),
-                       (getVideoFromVendor,(self.Get('hr_video'),list_video)))
+        if self.Get('os_install_x11_server_set') == 'on':
+            # get available videodriver list from install or configure distributive
+            list_video = self.Choice('os_install_x11_video_drv')
+            if not list_video:
+                return self.default_video
+            # if type system is usb-hdd then get detect video driver
+            if self.Get('os_install_root_type') == 'usb-hdd':
+                methods = ((getVideoFromModules,()),
+                           (getVideoFromCmdLine,()),
+                           (getVideoFromVendor,(self.Get('hr_video'),list_video)))
+            else:
+                # test current video driver for install system
+                methods = ((getVideoFromXorgLog,('/',list_video)),
+                           (getVideoFromXorgConf,('/',)),
+                           (getVideoFromModules,()),
+                           (getVideoFromCmdLine,()),
+                           (getVideoFromVendor,(self.Get('hr_video'),list_video)))
+            for func,args in methods:
+                drv = func(*args)
+                if drv in list_video:
+                    return drv
+            return self.default_video
         else:
-            # test current video driver for install system
-            methods = ((getVideoFromXorgLog,('/',list_video)),
-                       (getVideoFromXorgConf,('/',)),
-                       (getVideoFromModules,()),
-                       (getVideoFromCmdLine,()),
-                       (getVideoFromVendor,(self.Get('hr_video'),list_video)))
-        for func,args in methods:
-            drv = func(*args)
-            if drv in list_video:
-                return drv
-        return "other"
+            for drv in map(lambda x:x[0], self.choice()):
+                videoSysPath = path.join("/sys/module",drv,"refcnt")
+                refcnt = readFile(videoSysPath).strip()
+                if refcnt.isdigit() and int(refcnt) > 0:
+                    return {'i915':'intel'}.get(drv,drv)
+            else:
+                return self.default_video
 
     pkgDrvMap = {'nvidia':('NVidia','x11-drivers/nvidia-drivers'),
                  'fglrx':('ATI','x11-drivers/ati-drivers'),
                  'vboxdrv':('VirtualBox','x11-drivers/xf86-video-virtualbox')}
 
     def check(self,value):
-        if self.Get('cl_action') == 'system':
-            availDrvs = self.Get('os_install_x11_video_available')
-            if not value in availDrvs:
-                raise VariableError(_("Only %s drivers are available")%
-                        ",".join(availDrvs))
+        if self.Get('os_install_x11_server_set') == 'on':
+            if self.Get('cl_action') == 'system':
+                availDrvs = self.Get('os_install_x11_video_available')
+                if not value in availDrvs:
+                    raise VariableError(_("Only %s drivers are available")%
+                            ",".join(availDrvs))
+            else:
+                if not value in getInstalledVideo(prefix="/") and \
+                    not value in ("auto",self.default_video):
+                    error =_("video driver %s is unavailable")%value
+                    if value in self.pkgDrvMap:
+                        error += ". " + (_("Install driver %s with:")
+                                            %self.pkgDrvMap[value][0])
+                        error += "\n" + ("emerge %s"%self.pkgDrvMap[value][1])
+                    raise VariableError(error)
         else:
-            if not value in getInstalledVideo(prefix="/") and \
-                not value in ("auto","other"):
-                error =_("video driver %s is unavailable")%value
-                if value in self.pkgDrvMap:
-                    error += ". " + (_("Install driver %s with:")
-                                        %self.pkgDrvMap[value][0])
-                    error += "\n" + ("emerge %s"%self.pkgDrvMap[value][1])
-                raise VariableError(error)
+            availDrivers = self.Get('os_x11_kms_video_drv') + [self.default_video]
+            if not value in availDrivers:
+                raise VariableError("Only %s drivers are available" %
+                                    ",".join(availDrivers))
+
+    def uncompatible(self):
+        """
+        Video setting up unavailable for flash installation
+        """
+        if self.Get('os_install_root_type') == 'flash':
+            return \
+                _("Video configuration unavailable for Flash install")
+        return ""
 
 class VariableHrVideoId(ReadonlyVariable):
     """
@@ -258,21 +299,3 @@ class VariableOsInstallFbResolution(ResolutionVariable):
                     int(cxx11) >= int(cxfb) and int(cyx11) >= int(cyfb):
                     resolution = "%s-32"%textLines[0]
         return resolution or "1024x768-32"
-
-class VariableOsInstallX11ServerSet(ReadonlyVariable):
-    """
-    Is install xorg-server
-    """
-    type = "bool"
-
-    def get(self):
-        try:
-            image = self.Get('cl_image')
-            if image:
-                with image as distr:
-                    distrPath = image.getDirectory()
-                    if isPkgInstalled('xorg-server',prefix=distrPath):
-                        return "yes"
-        except:
-            pass
-        return "no"
